#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

void initialize()
{
	
	/* activam aproape toti pinii de pe PORTA si PORTB ca pini de iesire, deoarece acolo se afla conectate LED-urile */
	DDRA |= 255;
	DDRA &= ~(1 << PA6);
	DDRA &= ~(1 << PA7);
	
	DDRB |= 255;
	
	/* activam PD0 ca pin de intrare acolo aflandu-se switch-ul */
	DDRD &= ~(1 << PD0);
	PORTD |= (1 << PD0);
	
}


int state(int number)
{
	int i, j;
	
	/* modul normal de rulare al intersectiei are 9 stari */
	switch (number)
	{
		/* starea 0 este un rosu general de 2 secunde */
		case 0:
			PORTA = 0b00001001;
			PORTB = 0b00110011;
			
			break;
		
		/* starea 1 -> S1 verde, S2 rosu, P1 verde, P2 rosu si dureaza 12 secunde */
		case 1:
			PORTA = 0b00001100;
			PORTB = 0b00111100;
			
			break;
		
		/* starea 2 -> S1 verde, S2 rosu, P1 verde clipit, P2 rosu si dureaza 4 secunde */
		case 2:
			PORTA = 0b00001100;
			PORTB = 0b00111100;
			
			for (i = 0; i < 8; i ++)
			{
				PORTB ^= (1 << PB2);
				PORTB ^= (1 << PB3);
				
			}
			break;
		
		/* starea 3 -> S1 verde, S2 rosu, P1 rosu, P2 rosu si dureaza 4 secunde */
		case 3:
			PORTA = 0b00001100;
			PORTB = 0b00110011;
			
			for (i = 0; i < 4000; i ++)
			{
				_delay_ms(1);
			//	if (testSwitchChange() == 1)
					return 0;
			}
			break;
		
		/* starea 4 -> S1 galben, S2 rosu, P1 rosu, P2 rosu si dureaza 3 secunde */
		case 4:
			PORTA = 0b00001010;
			PORTB = 0b00110011;

			break;
		
		/* starea 5 este un rosu general de 2 secunde */
		case 5:
			PORTA = 0b00001001;
			PORTB = 0b00110011;
			
			break;
		
		/* starea 6 -> S1 rosu, S2 verde, P1 rosu, P2 verde si dureaza 12 secunde */
		case 6:
			PORTA = 0b00100001;
			PORTB = 0b11000011;
		
			break;
		
		/* starea 7 -> S1 rosu, S2 verde, P1 rosu, P2 verde clipit si dureaza 4 secunde */
		case 7:
			PORTA = 0b00100001;
			PORTB = 0b11000011;
			
			for (i = 0; i < 8; i ++)
			{
				PORTB ^= (1 << PB6);
				PORTB ^= (1 << PB7);
				
			}
			break;
		
		/* starea 8 -> S1 rosu, S2 verde, P1 rosu, P2 rosu si dureaza 4 secunde */
		case 8:
			PORTA = 0b00100001;
			PORTB = 0b00110011;
			
			for (i = 0; i < 4000; i ++)
			{
				_delay_ms(1);
				//if (testSwitchChange() == 1)
					return 0;
			}
			break;
		
		/* starea 9 -> S1 rosu, S2 galben, P1 rosu, P2 rosu si dureaza 4 secunde */
		case 9:
			PORTA = 0b00010001;
			PORTB = 0b00110011;
			
			break;
	}
	
	return 1;
}


void galben_intermitent()
{
	int i;
	
	/* inchidem toate LED-urile aprinse la momentul curent */
	PORTA &= 0;
	PORTB &= 0;
	
	while (1)
	{
		/* vom alterna starea LED-urilor galbene cand aprins cand oprit */
		PORTA ^= (1 << PA1);
		PORTA ^= (1 << PA4);
		
		/* odata la 500 milisecunde se va schimba starea LED-urilor galbene */
	}
}


void rulare_normala()
{
	int i;
	
	while (1)
		for (i = 0; i < 10; i ++)
			state(i);
}


int main(void)
{
    initialize();
	
	while (1)
		rulare_normala();
	
	
    return 0;
}
